<?php

namespace Drupal\field_inheritance\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\AppendCommand;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\ScrollTopCommand;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\field_inheritance\FieldInheritancePluginManager;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Provides an AJAX form for managing field inheritance entities.
 */
class FieldInheritanceAjaxForm extends FieldInheritanceForm {

  use AutowireTrait;

  /**
   * Construct a FieldInheritanceAjaxForm.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfo $entity_type_bundle_info
   *   The entity type bundle info service.
   * @param \Drupal\field_inheritance\FieldInheritancePluginManager $field_inheritance
   *   The field inheritance plugin manager.
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entityFormBuilder
   *   The entity form builder service.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   */
  public function __construct(
    MessengerInterface $messenger,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    #[Autowire(service: FieldInheritancePluginManager::class)]
    FieldInheritancePluginManager $field_inheritance,
    protected EntityFormBuilderInterface $entityFormBuilder,
    protected RendererInterface $renderer,
  ) {
    parent::__construct($messenger, $entity_type_manager, $entity_field_manager, $entity_type_bundle_info, $field_inheritance);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $form['actions']['submit']['#ajax'] = [
      'callback' => '::ajaxSubmit',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxSubmit(array $form, FormStateInterface $form_state): AjaxResponse {
    parent::save($form, $form_state);
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());

    // Grab the values from the form.
    $form_values = $form_state->getValues();

    // Build an empty entity using the destination entity type and bundle.
    $entity = $this->entityTypeManager->getStorage($form_values['destination_entity_type'])->create(['type' => $form_values['destination_entity_bundle']]);

    // Build the entity form for the created entity.
    $form = $this->entityFormBuilder->getForm($entity, 'default');

    // Grab the field inheritance field for the specified entity and add it to
    // the parent form.
    $field = $form['field_inheritance']['fields']['field_inheritance_' . $form_values['id']];
    $response->addCommand(new AppendCommand('#field-inheritance-ajax-container', $field));

    // Display the messages generated by the creation of the field inheritance.
    $message = [
      '#theme' => 'status_messages',
      '#message_list' => $this->messenger->all(),
    ];

    // Prevent duplicate messages appearing.
    $this->messenger->deleteAll();

    // Render the messages.
    $messages = $this->renderer->render($message);
    $response->addCommand(new HtmlCommand('#field-inheritance-ajax-message', $messages));
    $response->addCommand(new ScrollTopCommand('#field-inheritance-ajax-message'));
    return $response;
  }

}
