<?php

namespace Drupal\field_inheritance\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a form for configuring field inheritance settings.
 *
 * @ingroup field_inheritance
 */
class FieldInheritanceSettingsForm extends ConfigFormBase {

  use AutowireTrait;

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'field_inheritance_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['field_inheritance.config'];
  }

  /**
   * Construct a FieldInheritanceSettingsForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfo $entityTypeBundleInfo
   *   The entity type bundle info service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typedConfigManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('field_inheritance.config')
      ->set('included_entities', implode(',', array_filter($form_state->getValue('included_entities'))))
      ->set('included_bundles', implode(',', array_filter($form_state->getValue('included_bundles'))))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('field_inheritance.config');

    $entity_types = $this->entityTypeManager->getDefinitions();
    $entity_types = array_keys(array_filter($entity_types, function ($type) {
      return $type->entityClassImplements(FieldableEntityInterface::CLASS);
    }));
    $entity_types = array_combine($entity_types, $entity_types);

    $form['included_entities'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Included Entity Types'),
      '#description' => $this->t('Select the entity types that should be able to inherit data'),
      '#options' => $entity_types,
      '#default_value' => explode(',', $config->get('included_entities')),
    ];

    $entity_bundles = [];
    foreach ($entity_types as $entity_type) {
      $bundles = $this->entityTypeBundleInfo->getBundleInfo($entity_type);
      foreach (array_keys($bundles) as $bundle) {
        $entity_bundles[] = $entity_type . ':' . $bundle;
      }
    }
    $entity_bundles = array_combine($entity_bundles, $entity_bundles);

    $form['included_bundles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Included Entity Bundles'),
      '#description' => $this->t('Select the entity bundles that should be able to inherit data'),
      '#options' => $entity_bundles,
      '#default_value' => explode(',', $config->get('included_bundles')),
    ];

    return parent::buildForm($form, $form_state);
  }

}
