<?php

declare(strict_types=1);

namespace Drupal\field_inheritance\Hook;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldConfig;
use Drupal\field_inheritance\Entity\FieldInheritanceInterface;
use Drupal\field_inheritance\EntityReferenceFieldInheritanceFactory;
use Drupal\field_inheritance\FieldInheritanceFactory;
use Drupal\field_inheritance\FieldStorageDefinition;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Hook implementations for field_inheritance.
 */
class FieldInheritanceHooks {

  /**
   * Constructs a new FieldInheritanceHooks.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\KeyValueStore\KeyValueFactoryInterface $keyValue
   *   The keyvalue.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ModuleHandlerInterface $moduleHandler,
    protected EntityFieldManagerInterface $entityFieldManager,
    protected AccountInterface $currentUser,
    #[Autowire(service: 'keyvalue')]
    protected KeyValueFactoryInterface $keyValue,
    protected ConfigFactoryInterface $configFactory,
  ) {}

  /**
   * Implements hook_help().
   */
  #[Hook('help')]
  public function help(string $route_name, RouteMatchInterface $route_match): ?string {
    switch ($route_name) {
      // Main module help for the field_inheritance module.
      case 'help.page.field_inheritance':
        $output = '';
        return $output;
    }

    return NULL;
  }

  /**
   * Implements hook_entity_bundle_field_info_alter().
   */
  #[Hook('entity_bundle_field_info_alter')]
  public function entityBundleFieldInfoAlter(&$fields, EntityTypeInterface $entity_type, $bundle): void {
    $field_inheritance_storage = $this->entityTypeManager
      ->getStorage('field_inheritance');
    // Grab inherited fields just for the current entity type and bundle.
    $inherited_field_ids = $field_inheritance_storage
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('destinationEntityType', $entity_type->id())
      ->condition('destinationEntityBundle', $bundle)
      ->execute();

    if (!empty($inherited_field_ids)) {
      $inherited_fields = $field_inheritance_storage->loadMultiple($inherited_field_ids);
      if (!empty($inherited_fields)) {
        $x = 0;
        foreach ($inherited_fields as $field) {
          // We are only interested in adding computed fields to the destination
          // entity type.
          if ($field->destinationEntityType() !== $entity_type->id()) {
            continue;
          }

          // We are only interested in adding computed fields to the destination
          // entity bundle.
          if ($field->destinationEntityBundle() !== $bundle) {
            continue;
          }

          // Configure the settings array with all the values from the config
          // entity.
          $settings = [
            'id' => $field->idWithoutTypeAndBundle(),
            'source entity type' => $field->sourceEntityType(),
            'source entity bundle' => $field->sourceEntityBundle(),
            'source field' => $field->sourceField(),
            'method' => $field->type(),
            'plugin' => $field->plugin(),
          ];

          // Only set the destination field if one was configured, which will be
          // for inheritance strategies other than inherit.
          if ($field->destinationField()) {
            $settings['destination field'] = $field->destinationField();
          }

          // Grab the configuration of the source field.
          $source_field = FieldConfig::loadByName($field->sourceEntityType(), $field->sourceEntityBundle(), $field->sourceField());
          if (!empty($source_field)) {
            $settings = array_merge($settings, $source_field->getSettings());
            $type = $source_field->getType();
          }
          else {
            // This may be a base_field.
            $base_fields = $this->entityFieldManager->getBaseFieldDefinitions($field->sourceEntityType());
            if (!empty($base_fields[$field->sourceField()])) {
              $source_field = $base_fields[$field->sourceField()];
              $settings = array_merge($settings, $source_field->getSettings());
              $type = $source_field->getType();
            }
            else {
              continue;
            }
          }

          // Set the factory class used to inherit the data from the source.
          $class = $field->plugin() === 'entity_reference_inheritance' ?
            EntityReferenceFieldInheritanceFactory::class :
            FieldInheritanceFactory::class;

          // Allow developers to override the class to use for a field.
          $this->moduleHandler->alter('field_inheritance_inheritance_class', $class, $field);

          // Add a computed field for the inherited field to the appropriate
          // destination entity type and bundle.
          $fields[$field->idWithoutTypeAndBundle()] = FieldStorageDefinition::create($type)
            ->setLabel(t('@label', ['@label' => $field->label()]))
            ->setName($field->idWithoutTypeAndBundle())
            ->setDescription(t('The inherited field: @field', ['@field' => $field->label()]))
            ->setComputed(TRUE)
            ->setClass($class)
            ->setSettings($settings)
            ->setTargetEntityTypeId($field->destinationEntityType())
            ->setTargetBundle($field->destinationEntityBundle())
            ->setTranslatable(FALSE)
            ->setRevisionable(FALSE)
            ->setReadOnly(TRUE)
            ->setDisplayConfigurable('view', TRUE)
            ->setDisplayOptions('view', [
              'label' => 'visible',
              'weight' => (50 + $x),
            ])
            ->setProvider('field_inheritance');
          $x++;
        }
      }
    }
  }

  /**
   * Implements hook_form_alter().
   */
  #[Hook('form_alter')]
  public function formAlter(&$form, FormStateInterface $form_state, $form_id): void {
    if (!$this->currentUser->hasPermission('administer field inheritance')) {
      return;
    }
    $field_inheritance_storage = $this->entityTypeManager
      ->getStorage('field_inheritance');
    $form_object = $form_state->getFormObject();
    if (!empty($form_object) && $form_object instanceof EntityForm) {
      $entity = $form_state->getFormObject()->getEntity();
      if (!empty($entity) && $entity instanceof FieldableEntityInterface && strpos($form['#id'], 'delete') === FALSE) {
        $entity_type = $entity->getEntityTypeId();
        $bundle = $entity->bundle();

        $config = $this->configFactory->get('field_inheritance.config');
        $included_entities = $config->get('included_entities');
        $included_bundles = $config->get('included_bundles');

        if ($included_entities !== NULL) {
          $included_entities = explode(',', $included_entities);
          if (!in_array($entity_type, $included_entities)) {
            return;
          }
        }

        if ($included_bundles !== NULL) {
          $included_bundles = explode(',', $included_bundles);
          if (!in_array($entity_type . ':' . $bundle, $included_bundles)) {
            return;
          }
        }

        $state_key = $entity->getEntityTypeId() . ':' . $entity->uuid();
        $state = $this->keyValue->get('field_inheritance');
        $state_values = $state->get($state_key);

        $form['actions']['submit']['#submit'][] = 'field_inheritance_entity_form_submit';
        $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
        $form['field_inheritance'] = [
          '#type' => 'details',
          '#open' => TRUE,
          '#title' => t('Field Inheritance'),
          '#attributes' => [
            'id' => 'field-inheritance--wrapper',
          ],
        ];

        $form['field_inheritance']['field_inheritance_enable'] = [
          '#type' => 'checkbox',
          '#title' => t('Enable inheritance for this entity?'),
          '#description' => t('Enabling inheritance will allow data to be pulled in from a source entity into this entity.'),
          '#default_value' => $state_values['enabled'] ?? NULL,
        ];

        $form['field_inheritance']['fields'] = [
          '#type' => 'fieldset',
          '#title' => t('Field Mappings'),
          '#states' => [
            'visible' => [
              'input[name="field_inheritance_enable"]' => ['checked' => TRUE],
            ],
          ],
        ];

        $inherited_field_ids = $field_inheritance_storage
          ->getQuery()
          ->accessCheck(FALSE)
          ->condition('destinationEntityType', $entity_type)
          ->condition('destinationEntityBundle', $bundle)
          ->execute();

        if (!empty($inherited_field_ids)) {
          $inherited_fields = $field_inheritance_storage->loadMultiple($inherited_field_ids);

          if (!empty($inherited_fields)) {
            foreach ($inherited_fields as $field) {
              assert($field instanceof FieldInheritanceInterface);
              $form['field_inheritance']['fields']['field_inheritance_' . $field->idWithoutTypeAndBundle()] = [
                '#type' => 'details',
                '#open' => TRUE,
                '#title' => t('Entity Mapping for @label [@entity: @bundle - @field]', [
                  '@label' => $field->label(),
                  '@entity' => $field->sourceEntityType(),
                  '@bundle' => $field->sourceEntityBundle(),
                  '@field' => $field->sourceField(),
                ]),
                'field_inheritance_field_skip_' . $field->idWithoutTypeAndBundle() => [
                  '#type' => 'checkbox',
                  '#title' => t('Do not inherit this field'),
                  '#description' => t('Checking this box will prevent data from being inherited for this field on this entity.'),
                  '#default_value' => $state_values[$field->idWithoutTypeAndBundle()]['skip'] ?? FALSE,
                ],
                'field_inheritance_field_entity_' . $field->idWithoutTypeAndBundle() => [
                  '#type' => 'entity_autocomplete',
                  '#title' => t('Source Entity'),
                  '#description' => t('Enter the name of the %bundle from which to inherit data.', [
                    '%bundle' => $field->sourceEntityBundle(),
                  ]),
                  '#target_type' => $field->sourceEntityType(),
                  '#selection_settings' => ['target_bundles' => [$field->sourceEntityBundle()]],
                  '#default_value' => !empty($state_values[$field->idWithoutTypeAndBundle()]['entity']) ?
                  $this->entityTypeManager->getStorage($field->sourceEntityType())->load($state_values[$field->idWithoutTypeAndBundle()]['entity']) :
                  FALSE,
                  '#states' => [
                    'invisible' => [
                      'input[name="field_inheritance_field_skip_' . $field->idWithoutTypeAndBundle() . '"]' => ['checked' => TRUE],
                    ],
                  ],
                ],
              ];
            }
          }
        }

        $form['field_inheritance']['fields']['message'] = [
          '#type' => 'markup',
          '#markup' => '<div id="field-inheritance-ajax-message"></div>',
        ];

        $form['field_inheritance']['fields']['ajax_container'] = [
          '#type' => 'container',
          '#attributes' => [
            'id' => 'field-inheritance-ajax-container',
          ],
        ];

        $form['field_inheritance']['fields']['add'] = [
          '#type' => 'fieldset',
          '#access' => $this->currentUser->hasPermission('administer field inheritance'),
          'message' => [
            '#type' => 'markup',
            '#prefix' => '<p>',
            '#markup' => t('Add a new field inheritance field to this entity type and bundle.'),
            '#suffix' => '</p>',
          ],
          'add_field_inheritance' => [
            '#type' => 'link',
            '#title' => t('Add Field Inheritance'),
            '#url' => Url::fromRoute('field_inheritance.creation_form', [
              'entity_type' => $entity->getEntityTypeId(),
              'entity_bundle' => $entity->bundle(),
            ]),
            '#attributes' => [
              'class' => ['use-ajax', 'button', 'button--small'],
              'data-dialog-type' => 'modal',
              'data-dialog-options' => Json::encode(['width' => '60%']),
            ],
          ],
        ];
      }
    }
  }

}
